@@ -23,16 +23,17 @@ from ooflib.common.IO.GUI import gtkutils
 from ooflib.common.IO.GUI import quit
 from ooflib.common.IO.GUI import subWindow
 import code, sys, string
+
 import gi
 gi.require_version("Gtk", "3.0")
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import Pango
 
 tabspacing = 4
 
 # Preprocessing for some string stuff.
-ordinary_printable = string.digits + string.letters + string.punctuation + " "
+ordinary_printable = (string.digits + string.ascii_letters +
+                      string.punctuation + " ")
 KEYVAL_UP = Gdk.keyval_from_name('Up')
 KEYVAL_DOWN = Gdk.keyval_from_name('Down')
 KEYVAL_LEFT = Gdk.keyval_from_name('Left')
@@ -44,7 +45,6 @@ KEYVAL_CTRL_R = Gdk.keyval_from_name('Control_R')
 
 # TODO: Add handling for Page_Up, Page_Down, Home, End, Delete
 
-
 # Build the file menu outside of the instance, so it doesn't
 # have to be managed when instances come and go.
 
@@ -140,7 +140,9 @@ class GUIConsole(code.InteractiveConsole, subWindow.SubWindow):
 
         # File emulation attributes.
         self.old_stdout = sys.stdout
+        self.old_stderr = sys.stderr
         sys.stdout = self
+        sys.stderr = self
         self.softspace = 0
         self.mode="a"
         #
@@ -365,6 +367,7 @@ class GUIConsole(code.InteractiveConsole, subWindow.SubWindow):
     def local_destroy(self, gtkobj):
         global current_console
         sys.stdout = self.old_stdout
+        sys.stderr = self.old_stderr
         current_console = None
 
     # File-like functionality, allowing the console to stand
@@ -387,11 +390,11 @@ class GUIConsole(code.InteractiveConsole, subWindow.SubWindow):
     def tell(self):
         debug.mainthreadTest()
         # In a Python file object, tell() returns "an integer giving
-        # the file object’s current position in the file, measured in
+        # the file object's current position in the file, measured in
         # bytes from the beginning of the file".  However,
         # get_char_count() gets "the number of characters in the
         # buffer; note that characters and bytes are not the same, you
-        # can’t e.g. expect the contents of the buffer in string form
+        # can't e.g. expect the contents of the buffer in string form
         # to be this many bytes long".
         
         # This method will probably never be called, so it doesn't
@@ -424,7 +427,7 @@ mainmenu.OOF.Windows.Console.add_gui_callback(make_console)
 
 # Operate the console in "raw" mode -- writes out a prompt, then
 # returns whatever was typed from then until a new line was
-# encountered.
+# encountered.  (raw_input() in python2 is input() in python3.)
 
 def raw_input(prompt):
     global current_console
@@ -440,9 +443,9 @@ def raw_input(prompt):
         current_console.raw_result=None
         return result
     else:
-        return sys.modules['__main__'].__builtins__.raw_input(prompt)
+        return sys.modules['__main__'].__builtins__.input(prompt)
 
 
-# Over-ride the "raw_input" function as seen from the
+# Over-ride the "input" function as seen from the
 # OOF evaluation namespace.
-utils.OOFdefine('raw_input', raw_input)
+utils.OOFdefine('input', raw_input)
