@@ -40,6 +40,8 @@ if config.dimension() == 2:
 elif config.dimension() == 3:
     import ooflib.engine.skeleton3d as skeleton
 
+from ooflib.common.utils import stringjoin
+
 import sys
 
 OOFMenuItem = oofmenu.OOFMenuItem
@@ -494,7 +496,7 @@ def writeSkeleton(datafile, skelcontext):
             # data file, so that they're not written twice.
             excludeProps = {}
             for mat in msmatls:
-                for prop in mat.properties():
+                for prop in mat.properties:
                     excludeProps[prop.registration().name()] = prop
             materialmenu.writeMaterials(datafile, skelmatls, excludeProps)
             # Now assign Materials to Groups.
@@ -515,9 +517,7 @@ def writeSkeleton(datafile, skelcontext):
 
         # Point boundaries
         # sort keys to print in a consistent order
-        sortedKeys = skeleton.pointboundaries.keys()
-        sortedKeys.sort()
-        #for pbname, pbdy in skeleton.pointboundaries.items():
+        sortedKeys = sorted(skeleton.pointboundaries.keys())
         for pbname in sortedKeys:
             pbdy = skeleton.pointboundaries[pbname]
             datafile.startCmd(skelmenu.PointBoundary)
@@ -532,16 +532,14 @@ def writeSkeleton(datafile, skelcontext):
 
         # Edge boundaries
         # sort keys to print in a consistent order
-        sortedKeys = skeleton.edgeboundaries.keys()
-        sortedKeys.sort()
-        # for ebname, ebdy in skeleton.edgeboundaries.items():
+        sortedKeys = sorted(skeleton.edgeboundaries.keys())
         for ebname in sortedKeys:
             ebdy = skeleton.edgeboundaries[ebname]
             datafile.startCmd(skelmenu.EdgeBoundary)
             datafile.argument('skeleton', skelpath)
             datafile.argument('name', ebname)
             #Interface branch
-            if ebdy._sequenceable==1:
+            if ebdy._sequenceable:
                 edgeset = rearrangeEdges([
                     tuple([nodedict[x] for x in edge.get_nodes()])
                     for edge in ebdy.edges
@@ -582,23 +580,39 @@ def rearrangeEdges(edges):
 
     # Construct dictionaries of edges keyed by their starting and end points.
     tails = {}           
-    heads = {}         
-    for edge in edges:
+    heads = {}
+    minedge = 0                 # which edge starts at the min node index
+    for i, edge in enumerate(edges):
         heads[edge[0]] = edge
         tails[edge[1]] = edge
+        if edge[0] < edges[minedge][0]:
+            minedge = i
 
-    newedges = [edges[0]]
+    # Start with the edge with the lowest index at its start point.
+    # This has no effect on the result unless the edges form a loop.
+    # If they do form a loop, differences in the order of the items in
+    # the given edges list will affect the order of the edges in the
+    # saved file, and that can cause the test suite to fail.  The
+    # order of the items in the edges list depends on the order in
+    # which they're retrieved from a Python set, and can vary with the
+    # Python version.
+    newedges = [edges[minedge]]
 
     # Build the list of contiguous edges going backwards from the
-    # start point of edges[0].  This loop exits when either it runs
+    # start point of edges[minedge].  This loop exits when either it runs
     # out of tails and throws a key error, or when it loops back on
     # itself and the conditional fails.
-    lastpt = edges[0][0]
+    lastpt = edges[minedge][0]
     try:
+        ## TODO: This is doing a linear search, since newedges is a
+        ## list.  It would be better to remove items from heads and
+        ## tails as they're used, and stop when a connecting edge
+        ## isn't found.  It's probably not a high priority
+        ## optimization, though.  Boundaries aren't that big.
         while tails[lastpt] not in newedges:
-            next = tails[lastpt]
-            newedges.append(next)
-            lastpt = next[0]
+            nextedge = tails[lastpt]
+            newedges.append(nextedge)
+            lastpt = nextedge[0]
 
     except KeyError:
         pass
@@ -609,12 +623,12 @@ def rearrangeEdges(edges):
     # point of edges[0].  If the previous block found a loop, then
     # this block's while-conditional will fail, and it will do
     # nothing.
-    lastpt = edges[0][1]
+    lastpt = edges[minedge][1]
     try:
         while heads[lastpt] not in newedges:
-            next = heads[lastpt]
-            newedges.append(next)
-            lastpt = next[1]
+            nextedge = heads[lastpt]
+            newedges.append(nextedge)
+            lastpt = nextedge[1]
     except KeyError:
         pass
 
@@ -626,8 +640,9 @@ def rearrangeEdges(edges):
 ##########
 # ABAQUS #
 ##########
+
 import datetime
-import string
+
 def writeABAQUSfromSkeleton(filename, mode, skelcontext):
     skelcontext.begin_reading()
     try:
@@ -675,7 +690,7 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
         buffer+="** Materials defined by OOF2:\n"
         for matname, details in materiallist.items():
             buffer+="**   %s:\n" % (matname)
-            for prop in details.properties():
+            for prop in details.properties:
                 for param in prop.registration().params:
                     buffer+="**     %s: %s\n" % (param.name,param.value)
         debug.fmsg("wrote materials")
@@ -686,7 +701,7 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
         listbuf=["*NODE\n"]
         for node in skeleton.nodes:
             listbuf.append("%d, %s, %s\n" % (nodedict[node],node.position().x,node.position().y))
-        buffer+=string.join(listbuf,"")
+        buffer+=stringjoin(listbuf,"")
 
         # Only expecting 3 or 4 noded skeleton elements
         for numnodes in [3,4]:
@@ -697,13 +712,16 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
                     listbuf2=["%d" % (elementdict[el])]
                     for node in el.nodes:
                         listbuf2.append("%d" % (nodedict[node]))
-                    listbuf.append(string.join(listbuf2,", ")+"\n")
+                    listbuf.append(stringjoin(listbuf2,", ")+"\n")
             if len(listbuf)>1:
-                buffer+=string.join(listbuf,"")
+                buffer+=stringjoin(listbuf,"")
 
         debug.fmsg("wrote elements")
 
-        for group in skelcontext.nodegroups.groups:
+        # Sort groups for reproducibility in tests.
+        grpnames = list(skelcontext.nodegroups.groups)
+        grpnames.sort()
+        for group in grpnames:
             buffer+="*NSET, NSET=%s\n" % (group)
             listbuf=[]
             i=0
@@ -713,10 +731,12 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
                 else:
                     listbuf.append("%d" % (nodedict[node]))
                 i+=1
-            buffer+=string.join(listbuf,", ")+"\n"
+            buffer+=stringjoin(listbuf,", ")+"\n"
         debug.fmsg("wrote node groups")
-        
-        for elgroup in skelcontext.elementgroups.groups:
+
+        grpnames = list(skelcontext.elementgroups.groups)
+        grpnames.sort()
+        for elgroup in grpnames:
             buffer+="*ELSET, ELSET=%s\n" % (elgroup)
             listbuf=[]
             i=0
@@ -726,11 +746,13 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
                 else:
                     listbuf.append("%d" % (elementdict[el]))
                 i+=1
-            buffer+=string.join(listbuf,", ")+"\n"
+            buffer+=stringjoin(listbuf,", ")+"\n"
         debug.fmsg("wrote element groups")
 
+        grps = list(skeleton.pointboundaries.items())
+        grps.sort()
         buffer+="** Include point and edge boundaries from OOF2.\n"
-        for pbname, pbdy in skeleton.pointboundaries.items():
+        for pbname, pbdy in grps:
             buffer+="*NSET, NSET=%s\n" % (pbname)
             listbuf=[]
             i=0
@@ -740,14 +762,16 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
                 else:
                     listbuf.append("%d" % (nodedict[node]))
                 i+=1
-            buffer+=string.join(listbuf,", ")+"\n"
+            buffer+=stringjoin(listbuf,", ")+"\n"
         debug.fmsg("wrote boundaries")
 
         # Use rearrangeEdges() to chain the edges together, then pick the
         #  unique nodes. It seems the edges can't be selected if they
         #  are empty, so edgeset=[(a,b),(b,c),...] is not checked
         #  for null content
-        for ebname, ebdy in skeleton.edgeboundaries.items():
+        bdys = list(skeleton.edgeboundaries.items())
+        bdys.sort()
+        for ebname, ebdy in bdys:
             edgeset = rearrangeEdges([
                 tuple([nodedict[node] for node in edge.get_nodes()])
                 for edge in ebdy.edges
@@ -761,7 +785,7 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
                 else:
                     listbuf.append("%d" % (edge[1]))
                 i+=1
-            buffer+=string.join(listbuf,", ")+"\n"
+            buffer+=stringjoin(listbuf,", ")+"\n"
 
         for matname in materiallist:
             buffer+="*ELSET, ELSET=%s\n" % matname
@@ -773,7 +797,7 @@ def writeABAQUSfromSkeleton(filename, mode, skelcontext):
                 else:
                     listbuf.append("%d" % (elindex))
                 i+=1
-            buffer+=string.join(listbuf,", ")+"\n*SOLID SECTION, ELSET=%s, MATERIAL=%s\n" % (matname,matname)
+            buffer+=stringjoin(listbuf,", ")+"\n*SOLID SECTION, ELSET=%s, MATERIAL=%s\n" % (matname,matname)
         debug.fmsg("wrote material elsets")
             
         for matname in materiallist:
