@@ -46,8 +46,8 @@ from ooflib.common.IO import parameter
 from ooflib.common.IO import xmlmenudump
 from ooflib.SWIG.engine import outputval
 import math
-import string
-import types
+
+from functools import reduce
 
 # Sample objects should have a "columnNames" attribute, which is a
 # list of strings which identify the columns of data which are
@@ -116,7 +116,7 @@ class ElementSample(Sample):
     #     return self.index
     def integrate(self, domain, output, order, power=1):
         femesh = domain.meshctxt.getObject()
-        gauss_pts = self.element.integration_points(order)
+        gauss_pts = list(self.element.integrator(order))
         pts = [x.mastercoord() for x in gauss_pts]
         wgts = [x.weight() for x in gauss_pts]
         if power==0:
@@ -131,7 +131,7 @@ class ElementSample(Sample):
                 return reduce(lambda x,y: x+y[0]*(y[1]**power),
                               zip(wgts, vals),
                               vals[0].zero())
-        raise ooferror.ErrPyProgrammingError("Impossible situation arose")
+        raise ooferror.PyErrPyProgrammingError("Impossible situation arose")
 
     # The ElementSample is actually not used for direct output, but
     # can use this mechanism for debugging purposes.
@@ -147,7 +147,7 @@ class PointSample(Sample):
     # def identifier(self):
     #     return self.point
     def outputData(self):
-        return [`self.point.x`, `self.point.y`]
+        return [repr(self.point.x), repr(self.point.y)]
 
 
 
@@ -163,8 +163,8 @@ class LineSample(PointSample):
     # def identifier(self):
     #     return (self.point, self.fraction, self.distance)
     def outputData(self):
-        return [`self.distance`, `self.fraction`,
-                `self.point.x`, `self.point.y`] 
+        return [repr(self.distance), repr(self.fraction),
+                repr(self.point.x), repr(self.point.y)] 
 
 
 # A portion of a line contained within one element.  "Segment" is a
@@ -212,8 +212,8 @@ class ElementLineSample(Sample):
             dx = df = dd = 0
         for i in range(self.n_points):
             lab_pt = start+i*dx
-            outputData = [`self.segment`, `self.distances[0]+i*dd`,
-                          `self.fractions[0]+i*df`, `lab_pt[0]`, `lab_pt[1]`]
+            outputData = [repr(self.segment), repr(self.distances[0]+i*dd),
+                          repr(self.fractions[0]+i*df), repr(lab_pt[0]), repr(lab_pt[1])]
             out_list.append( [ outputData[self.columnNames.index(x)]
                               for x in header] )
         return out_list
@@ -233,7 +233,7 @@ class PixelSample(Sample):
     #     return self.pixel
     def outputData(self):
         return [ "(%d %d)" % (self.pixel[0], self.pixel[1]),
-                 `self.point[0]`, `self.point[1]` ]
+                 repr(self.point[0]), repr(self.point[1]) ]
     # For integration and direct output, pixels are treated as points
     # at their centers.  It may be desirable, at some point, to take
     # the integration operation more seriously.
@@ -310,7 +310,7 @@ class SampleSet(registeredclass.RegisteredClass):
 # an object attribute with the same name.
 
 def _attr_from_colname(colname):
-    return "show_"+string.lower(colname)
+    return "show_"+colname.lower()
 
 
 #=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#=-=#
@@ -324,9 +324,11 @@ class ElementSampleSet(SampleSet):
     def make_samples(self, domain):
         self.sample_list = []
         els = domain.get_elements()
+        didsomething = False
         for e in els:
+            didsomething = True
             self.sample_list.append(ElementSample(e))
-        return len(els) > 0
+        return didsomething
     def integrate(self, domain, output, power=1):
         if self.order==automatic.automatic:
             order=2            # TODO: Do something cleverer here.
@@ -334,7 +336,7 @@ class ElementSampleSet(SampleSet):
             order = self.order
         vals = [x.integrate(domain, output, order, power)
                 for x in self.sample_list]
-        return zip(self.sample_list, vals)
+        return list(zip(self.sample_list, vals))
     def get_col_names(self):    # see comment in SampleSet.
         return []
 
@@ -511,7 +513,7 @@ class SpacedGridSampleSet(PointSampleSet):
             bxmax = box.xmax()
             bymax = box.ymax()
             (x,y)=(bxmin,bymin)
-            while 1:
+            while True:
                 pt = primitives.Point(x,y)
                 if domain.contains(pt):
                     self.sample_list.append(PointSample(pt))
@@ -527,10 +529,10 @@ DirectSampleSetRegistration(
     "Spaced Grid Points",
     SpacedGridSampleSet,
     21,
-    params=[parameter.FloatParameter(
+    params=[parameter.PositiveFloatParameter(
             'delta_x', 0.1,
             tip="Horizontal spacing between grid points."),
-            parameter.FloatParameter(
+            parameter.PositiveFloatParameter(
             'delta_y', 0.1,
             tip="Vertical spacing between grid points.")],
     sample_type=GRID,
@@ -610,7 +612,7 @@ class ElementSegmentSampleSet(SampleSet):
     containedClass = ElementLineSample
     def __init__(self, n_points, **kwargs):
         if n_points < 2:
-            raise ooferror.ErrUserError("n_points must be greater than 1!")
+            raise ooferror.PyErrUserError("n_points must be greater than 1!")
         self.n_points = n_points
         SampleSet.__init__(self, **kwargs)
     def make_samples(self, domain):
@@ -653,7 +655,7 @@ class ElementSegmentSampleSet(SampleSet):
                     rval += (vals[i]+vals[i+1])*dx/2.0
                 result.append(rval)
 
-        return zip(self.sample_list, result)
+        return list(zip(self.sample_list, result))
 
             
 DirectSampleSetRegistration(
@@ -690,7 +692,7 @@ class PixelSampleSet(SampleSet):
     def integrate(self, domain, output, power=1):
         vals = [x.integrate(domain, output, power) 
                 for x in self.sample_list]
-        return zip(self.sample_list, vals)
+        return list(zip(self.sample_list, vals))
     
 
 DirectSampleSetRegistration(
