@@ -9,7 +9,7 @@
 # oof_manager@nist.gov. 
 
 from ooflib.SWIG.common import switchboard
-from ooflib.SWIG.engine import ooferror2
+from ooflib.SWIG.engine import ooferror
 from ooflib.common import debug
 from ooflib.common import enum
 from ooflib.common import utils
@@ -84,6 +84,16 @@ registeredclass.Registration(
 # The Schedule class determines when scheduled output is produced.
 # UnconditionalSchedules determine the output times in advance.
 # ConditionalSchedules do it on the fly.
+
+# Subclasses of Schedule need to have a __next__() method which returns
+# the next time in the schedule, or raises StopIteration if there is
+# no next time.  Schedule.__next__() is called by
+# OutputSchedule.reset() and OutputSchedule.times(), below, via
+# next().
+
+# The subclasses should also have a reset() method.  After reset() is
+# called, the next time returned by __next__() should be the first
+# time.
     
 class Schedule(registeredclass.RegisteredClass):
     registry = []
@@ -134,7 +144,7 @@ class Once(UnconditionalSchedule):
         UnconditionalSchedule.__init__(self)
     def reset(self, continuing):
         self.done = False
-    def next(self):
+    def __next__(self):
         if self.done:
             raise StopIteration
         self.done = True
@@ -159,7 +169,7 @@ class Periodic(UnconditionalSchedule):
         UnconditionalSchedule.__init__(self)
     def reset(self, continuing):
         self.count = 0
-    def next(self):
+    def __next__(self):
         t = self.time0 + self.delay + self.count*self.interval
         self.count += 1
         return t
@@ -189,7 +199,7 @@ class Geometric(UnconditionalSchedule):
         if not continuing:
             self.nextstep = self.timestep
             self.lasttime = None
-    def next(self):
+    def __next__(self):
         ## TODO: This doesn't quite do the right thing on the first
         ## step of a continued computation, if the previous step was
         ## truncated to hit the end time exactly.  For example, with
@@ -229,7 +239,7 @@ class SpecifiedTimes(UnconditionalSchedule):
     def reset(self, continuing):
         if not continuing:
             self.count = 0
-    def next(self):
+    def __next__(self):
         if self.count == len(self.times):
             raise StopIteration
         t = self.times[self.count] + self.time0
@@ -269,7 +279,7 @@ registeredclass.Registration(
 
 ## TODO: can nextoutputs just be a list?  Why an OrderedSet?
 
-class OutputSchedule(object):
+class OutputSchedule:
     def __init__(self, meshcontext):
         # outputs is a set of ScheduledOutput objects.  nexttimes is
         # a dictionary, keyed by ScheduledOutput objects, giving the
@@ -306,7 +316,7 @@ class OutputSchedule(object):
                 # output. If it is, silently delete it.
                 try:
                     output.setDestination(o.destination)
-                except ooferror2.ErrInvalidDestination:
+                except ooferror.PyErrInvalidDestination:
                     pass
                 # If the old output was given an automatically
                 # generated name, the name must be updated to reflect
@@ -373,13 +383,13 @@ class OutputSchedule(object):
                     self.conditionalOutputs.add(output)
                 else:
                     try:
-                        t = roundOffCheck(output.schedule.next(), time0)
+                        t = roundOffCheck(next(output.schedule), time0)
                         if continuing:
                             while t <= time0:
-                                t = output.schedule.next()
+                                t = next(output.schedule)
                         else:
                             while t < time0:
-                                t = output.schedule.next()
+                                t = next(output.schedule)
                     except StopIteration:
                         # The schedule has no times in it later than time0.
                         # Just ignore it.
@@ -406,7 +416,7 @@ class OutputSchedule(object):
             for output in self.nextoutputs:
                 try:
                     self.nexttimes[output] = roundOffCheck(
-                        output.schedule.next(), endtime)
+                        next(output.schedule), endtime)
                 except StopIteration:
                     del self.nexttimes[output]
                     self.finished.add(output)
