@@ -14,7 +14,7 @@ from ooflib.SWIG.common import switchboard
 from ooflib.SWIG.common import timestamp
 from ooflib.SWIG.engine import field
 from ooflib.SWIG.engine import flux
-from ooflib.SWIG.engine import ooferror2
+from ooflib.SWIG.engine import ooferror
 from ooflib.SWIG.engine import sparsemat
 from ooflib.common import debug
 from ooflib.common import utils
@@ -26,7 +26,6 @@ from ooflib.engine import meshstatus
 from ooflib.engine import nonlinearsolver
 from ooflib.engine import timestepper
 from ooflib.engine import solverstats
-import string
 import sys
 import datetime
 
@@ -226,7 +225,7 @@ class SubProblemContext(whoville.Who):
     def preremoveCB(self, path):
         # If one of our dependencies is being removed, we're done for.
         # We might as well end it all right now.
-        path = string.join(path, ':')
+        path = utils.stringjoin(path, ':')
         if path in self.subptype.get_dependencies():
             self.reserve()
             self.begin_writing()
@@ -256,20 +255,18 @@ class SubProblemContext(whoville.Who):
         return self.getParent().getMicrostructure()
 
     def nelements(self):
-        return self.getObject().element_iterator().size()
+        return self.getObject().elements().size()
 
     def nnodes(self):
-        return self.getObject().node_iterator().size()
+        return self.getObject().nodes().size()
 
     def nfuncnodes(self):
-        return self.getObject().funcnode_iterator().size()
+        return self.getObject().funcnodes().size()
 
     def area(self):
         a = 0.0
-        iter = self.getObject().element_iterator()
-        while not iter.end():
-            a += iter.element().area()
-            iter.next()
+        for element in self.elements():
+            a += element.area()
         return a
 
     def solved(self):
@@ -322,10 +319,7 @@ class SubProblemContext(whoville.Who):
 
     def all_fields(self):
         compound_fields = self.all_compound_fields()
-        if config.dimension() == 2:
-            zfields = [fld.out_of_plane() for fld in compound_fields]
-        else:
-            zfields = []
+        zfields = [fld.out_of_plane() for fld in compound_fields]
         tfields = [fld.time_derivative() for fld in compound_fields]
         tfields = [t for t in tfields if self.is_defined_field(t)]
         return compound_fields + zfields + tfields
@@ -426,7 +420,7 @@ class SubProblemContext(whoville.Who):
                     unsolvable = self.checkSolvability()
                     if unsolvable:
                         if solving:
-                            raise ooferror2.ErrUserError(unsolvable)
+                            raise ooferror.PyErrUserError(unsolvable)
                         else:
                             return
 
@@ -493,7 +487,7 @@ class SubProblemContext(whoville.Who):
             if self.is_active_equation(eqn):
                 for mat in self.getObject().getMaterials():
                     order = 0
-                    for prop in mat.properties():
+                    for prop in mat.properties:
                         reg = prop.registration()
                         if reg.second_order_fields(eqn):
                             order = 2
@@ -597,7 +591,7 @@ class SubProblemContext(whoville.Who):
         # The linsys argument is either a LinearizedSystem object
         # previously created by this SubProblemContext, or None.  If
         # it's not None, it will be updated and reused.
-	utils.memusage("Make Linear System Started %s" %datetime.datetime.now())
+        # utils.memusage("Make Linear System Started %s" %datetime.datetime.now())
         mesh    = self.getParent()
         femesh  = mesh.getObject()
         subpobj = self.getObject()
@@ -623,7 +617,7 @@ class SubProblemContext(whoville.Who):
         ## timestamps in the Mesh? 
 
         femesh.setCurrentSubProblem(self.getObject())
-	utils.memusage("Completed Set FE Mesh %s" %datetime.datetime.now())
+        # utils.memusage("Completed Set FE Mesh %s" %datetime.datetime.now())
 
         # Figure out which parts of the calculation have to be redone.
         # If always is set, all steps of the calculation will be
@@ -688,14 +682,14 @@ class SubProblemContext(whoville.Who):
         if newTime:
             linsys.set_time(time)
 
-	utils.memusage("Start Setting Boundary Conditions %s" %datetime.datetime.now())
+        # utils.memusage("Start Setting Boundary Conditions %s" %datetime.datetime.now())
 
         # Apply boundary conditions to the linearized system object.
         # This has to be done before the matrix and rhs values are
         # computed. The matrix and rhs may be nonlinear and therefore
         # depend on field values, which may be determined by boundary
         # conditions.
-            
+
         bcsReset = newLinSys or newBdys
         if bcsReset:
             # reset_bcs() calls Boundary.reset for all Boundaries in
@@ -719,21 +713,21 @@ class SubProblemContext(whoville.Who):
 
             femesh.invoke_flux_bcs(subpobj, linsys, time)
 
-	utils.memusage("Start Apply Dirichlet Boundary Conditions %s" %datetime.datetime.now())
+        # utils.memusage("Start Apply Dirichlet Boundary Conditions %s" %datetime.datetime.now())
         # Apply Dirichlet BCs.  If new values have been assigned to
         # the Fields, the old Dirichlet BCs may have been overwritten,
         # so they have to be reapplied.
         if bcsReset or newFieldValues:
             linsys.resetFieldFlags()
             femesh.invoke_fixed_bcs(subpobj, linsys, time)
-	utils.memusage("End if bcsReset or newFieldValues %s" %datetime.datetime.now())
+        # utils.memusage("End if bcsReset or newFieldValues %s" %datetime.datetime.now())
 
         if bcsReset:
             femesh.invoke_force_bcs(subpobj, linsys, time)
             # Set initial values of DoFs used in FloatBCs that
             # intersect fixedBCs.
             femesh.fix_float_bcs(subpobj, linsys, time)
-	utils.memusage("End if bcsReset %s" %datetime.datetime.now())
+        # utils.memusage("End if bcsReset %s" %datetime.datetime.now())
 
         if rebuildMatrices:
             # Assemble vectors and matrices of the linearized system.
@@ -744,31 +738,31 @@ class SubProblemContext(whoville.Who):
             if self.nonlinear_solver.needsJacobian():
                 linsys.clearJacobian()
             # **** This is the cpu intensive step: ****
-            utils.memusage("Start Make linear system %s" %datetime.datetime.now())
+            # utils.memusage("Start Make linear system %s" %datetime.datetime.now())
             self.getObject().make_linear_system(linsys, self.nonlinear_solver)
-	    utils.memusage("End Make linear system %s" %datetime.datetime.now())
+            # utils.memusage("End Make linear system %s" %datetime.datetime.now())
 
             self.newMatrixCount += 1
-	utils.memusage("End if rebuildMatricies %s" %datetime.datetime.now())
+        # utils.memusage("End if rebuildMatricies %s" %datetime.datetime.now())
 
         if bcsReset or rebuildMatrices or newFieldValues:
-	    utils.memusage("Begin Build submatrix maps %s" %datetime.datetime.now())
+            # utils.memusage("Begin Build submatrix maps %s" %datetime.datetime.now())
             linsys.build_submatrix_maps()
-	    utils.memusage("End Build submatrix maps %s" %datetime.datetime.now())
+            # utils.memusage("End Build submatrix maps %s" %datetime.datetime.now())
             # Apply floating boundary conditions by modifying maps in
             # the LinearizedSystem.  This must follow
             # build_submatrix_maps() and precede build_MCK_maps().
-	    utils.memusage("Begin invoke flot bcs %s" %datetime.datetime.now())
+            # utils.memusage("Begin invoke float bcs %s" %datetime.datetime.now())
             femesh.invoke_float_bcs(subpobj, linsys, time)
-	    utils.memusage("Begin Build MCK maps %s" %datetime.datetime.now())
+            # utils.memusage("Begin Build MCK maps %s" %datetime.datetime.now())
             linsys.build_MCK_maps()
-	    utils.memusage("End Build MCK maps %s" %datetime.datetime.now())
+            # utils.memusage("End Build MCK maps %s" %datetime.datetime.now())
             # Construct vectors of first and second time derivatives
             # of the time-dependent Dirichlet boundary conditions.
             linsys.initDirichletDerivatives()
-	    utils.memusage("Begin set Dirichlet Derivatives %s" %datetime.datetime.now())
+            # utils.memusage("Begin set Dirichlet Derivatives %s" %datetime.datetime.now())
             femesh.setDirichletDerivatives(subpobj, linsys, time)
-	utils.memusage("End if bcsReset or rebuildMatricies or newFieldValues %s" %datetime.datetime.now())
+        # utils.memusage("End if bcsReset or rebuildMatricies or newFieldValues %s" %datetime.datetime.now())
 
 
         if bcsReset:
@@ -783,7 +777,7 @@ class SubProblemContext(whoville.Who):
             ## TODO OPT: Only call float_contrib_rhs if solver needs
             ## to know the rhs explicitly.
             femesh.float_contrib_rhs(self.getObject(), linsys)
-	utils.memusage("End Setting Boundary Conditions %s" %datetime.datetime.now())
+        # utils.memusage("End Setting Boundary Conditions %s" %datetime.datetime.now())
 
         femesh.clearCurrentSubProblem()
         linsys.computed.increment()
@@ -839,12 +833,12 @@ class SubProblemContext(whoville.Who):
             self.nonlinear_solver.computeStaticFields(self, linsys, unknowns)
                 
     def computeStaticFieldsL(self, linsys, unknowns):
-        utils.memusage("start of computeStaticFieldsL")
+        # utils.memusage("start of computeStaticFieldsL")
         # Initialize "static" fields for linear problems. 
         if linsys.n_unknowns_part('K')==0 and linsys.n_unknowns_part('C')==0:
             return
         if self.nonlinear_activefields():
-            raise ooferror2.ErrSetupError(
+            raise ooferror.PyErrSetupError(
                 "A nonlinear solver is required for subproblem '%s'." 
                 % self.name())
 
@@ -878,11 +872,11 @@ class SubProblemContext(whoville.Who):
             if len(u2) > 0:
                 K02 = self.time_stepper.K_submatrix(linsys, 'K', 'M')
                 rhs -= K02*u2
-	    utils.memusage("Before Matrix Method Solve")
+            # utils.memusage("Before Matrix Method Solve")
             self.matrix_method(self.asymmetricK).solve(K00, rhs, u0)
-	    utils.memusage("After Matrix Method Solve")
+            # utils.memusage("After Matrix Method Solve")
             self.time_stepper.set_unknowns_part('K', linsys, u0, unknowns)
-            utils.memusage("K computed")
+            # utils.memusage("K computed")
 
         if len(u1) > 0 and self.time_stepper.derivOrder() > 1:
             # Find u1dot by solving
@@ -1105,7 +1099,7 @@ class SubProblemContext(whoville.Who):
 
 ## TODO TDEP: Should this be derived from timestepper.NLData?
 
-class StaticNLFuncs(object):
+class StaticNLFuncs:
     def __init__(self, unknowns):
         self.unknowns = unknowns
 
@@ -1175,7 +1169,7 @@ consistencyTolerance = 1.e-6
 # can use the MatrixMethod classes independently of the
 # SubProblemContext if necessary.
 
-class MatrixSolverWrapper(object):
+class MatrixSolverWrapper:
     def __init__(self, subproblemcontext, solver):
         self.subprobctxt = subproblemcontext
         self.solver = solver
